# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ModelKernelClusterNode(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cpu_allocatable': 'float',
        'cpu_limits': 'float',
        'cpu_requests': 'float',
        'created_dt': 'datetime',
        'edges': 'ModelKernelClusterNodeEdges',
        'ephemeral_storage_allocatable': 'int',
        'ephemeral_storage_limits': 'int',
        'ephemeral_storage_requests': 'int',
        'gpu_allocatable': 'int',
        'gpu_limits': 'int',
        'gpu_product_name': 'str',
        'gpu_requests': 'int',
        'id': 'int',
        'memory_allocatable': 'int',
        'memory_limits': 'int',
        'memory_requests': 'int',
        'name': 'str',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'cpu_allocatable': 'cpu_allocatable',
        'cpu_limits': 'cpu_limits',
        'cpu_requests': 'cpu_requests',
        'created_dt': 'created_dt',
        'edges': 'edges',
        'ephemeral_storage_allocatable': 'ephemeral_storage_allocatable',
        'ephemeral_storage_limits': 'ephemeral_storage_limits',
        'ephemeral_storage_requests': 'ephemeral_storage_requests',
        'gpu_allocatable': 'gpu_allocatable',
        'gpu_limits': 'gpu_limits',
        'gpu_product_name': 'gpu_product_name',
        'gpu_requests': 'gpu_requests',
        'id': 'id',
        'memory_allocatable': 'memory_allocatable',
        'memory_limits': 'memory_limits',
        'memory_requests': 'memory_requests',
        'name': 'name',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, cpu_allocatable=None, cpu_limits=None, cpu_requests=None, created_dt=None, edges=None, ephemeral_storage_allocatable=None, ephemeral_storage_limits=None, ephemeral_storage_requests=None, gpu_allocatable=None, gpu_limits=None, gpu_product_name=None, gpu_requests=None, id=None, memory_allocatable=None, memory_limits=None, memory_requests=None, name=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """ModelKernelClusterNode - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cpu_allocatable = None
        self._cpu_limits = None
        self._cpu_requests = None
        self._created_dt = None
        self._edges = None
        self._ephemeral_storage_allocatable = None
        self._ephemeral_storage_limits = None
        self._ephemeral_storage_requests = None
        self._gpu_allocatable = None
        self._gpu_limits = None
        self._gpu_product_name = None
        self._gpu_requests = None
        self._id = None
        self._memory_allocatable = None
        self._memory_limits = None
        self._memory_requests = None
        self._name = None
        self._updated_dt = None
        self.discriminator = None

        self.cpu_allocatable = cpu_allocatable
        self.cpu_limits = cpu_limits
        self.cpu_requests = cpu_requests
        self.created_dt = created_dt
        if edges is not None:
            self.edges = edges
        self.ephemeral_storage_allocatable = ephemeral_storage_allocatable
        self.ephemeral_storage_limits = ephemeral_storage_limits
        self.ephemeral_storage_requests = ephemeral_storage_requests
        self.gpu_allocatable = gpu_allocatable
        self.gpu_limits = gpu_limits
        self.gpu_product_name = gpu_product_name
        self.gpu_requests = gpu_requests
        if id is not None:
            self.id = id
        self.memory_allocatable = memory_allocatable
        self.memory_limits = memory_limits
        self.memory_requests = memory_requests
        if name is not None:
            self.name = name
        self.updated_dt = updated_dt

    @property
    def cpu_allocatable(self):
        """Gets the cpu_allocatable of this ModelKernelClusterNode.  # noqa: E501


        :return: The cpu_allocatable of this ModelKernelClusterNode.  # noqa: E501
        :rtype: float
        """
        return self._cpu_allocatable

    @cpu_allocatable.setter
    def cpu_allocatable(self, cpu_allocatable):
        """Sets the cpu_allocatable of this ModelKernelClusterNode.


        :param cpu_allocatable: The cpu_allocatable of this ModelKernelClusterNode.  # noqa: E501
        :type cpu_allocatable: float
        """

        self._cpu_allocatable = cpu_allocatable

    @property
    def cpu_limits(self):
        """Gets the cpu_limits of this ModelKernelClusterNode.  # noqa: E501


        :return: The cpu_limits of this ModelKernelClusterNode.  # noqa: E501
        :rtype: float
        """
        return self._cpu_limits

    @cpu_limits.setter
    def cpu_limits(self, cpu_limits):
        """Sets the cpu_limits of this ModelKernelClusterNode.


        :param cpu_limits: The cpu_limits of this ModelKernelClusterNode.  # noqa: E501
        :type cpu_limits: float
        """

        self._cpu_limits = cpu_limits

    @property
    def cpu_requests(self):
        """Gets the cpu_requests of this ModelKernelClusterNode.  # noqa: E501


        :return: The cpu_requests of this ModelKernelClusterNode.  # noqa: E501
        :rtype: float
        """
        return self._cpu_requests

    @cpu_requests.setter
    def cpu_requests(self, cpu_requests):
        """Sets the cpu_requests of this ModelKernelClusterNode.


        :param cpu_requests: The cpu_requests of this ModelKernelClusterNode.  # noqa: E501
        :type cpu_requests: float
        """

        self._cpu_requests = cpu_requests

    @property
    def created_dt(self):
        """Gets the created_dt of this ModelKernelClusterNode.  # noqa: E501


        :return: The created_dt of this ModelKernelClusterNode.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ModelKernelClusterNode.


        :param created_dt: The created_dt of this ModelKernelClusterNode.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def edges(self):
        """Gets the edges of this ModelKernelClusterNode.  # noqa: E501


        :return: The edges of this ModelKernelClusterNode.  # noqa: E501
        :rtype: ModelKernelClusterNodeEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this ModelKernelClusterNode.


        :param edges: The edges of this ModelKernelClusterNode.  # noqa: E501
        :type edges: ModelKernelClusterNodeEdges
        """

        self._edges = edges

    @property
    def ephemeral_storage_allocatable(self):
        """Gets the ephemeral_storage_allocatable of this ModelKernelClusterNode.  # noqa: E501


        :return: The ephemeral_storage_allocatable of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._ephemeral_storage_allocatable

    @ephemeral_storage_allocatable.setter
    def ephemeral_storage_allocatable(self, ephemeral_storage_allocatable):
        """Sets the ephemeral_storage_allocatable of this ModelKernelClusterNode.


        :param ephemeral_storage_allocatable: The ephemeral_storage_allocatable of this ModelKernelClusterNode.  # noqa: E501
        :type ephemeral_storage_allocatable: int
        """

        self._ephemeral_storage_allocatable = ephemeral_storage_allocatable

    @property
    def ephemeral_storage_limits(self):
        """Gets the ephemeral_storage_limits of this ModelKernelClusterNode.  # noqa: E501


        :return: The ephemeral_storage_limits of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._ephemeral_storage_limits

    @ephemeral_storage_limits.setter
    def ephemeral_storage_limits(self, ephemeral_storage_limits):
        """Sets the ephemeral_storage_limits of this ModelKernelClusterNode.


        :param ephemeral_storage_limits: The ephemeral_storage_limits of this ModelKernelClusterNode.  # noqa: E501
        :type ephemeral_storage_limits: int
        """

        self._ephemeral_storage_limits = ephemeral_storage_limits

    @property
    def ephemeral_storage_requests(self):
        """Gets the ephemeral_storage_requests of this ModelKernelClusterNode.  # noqa: E501


        :return: The ephemeral_storage_requests of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._ephemeral_storage_requests

    @ephemeral_storage_requests.setter
    def ephemeral_storage_requests(self, ephemeral_storage_requests):
        """Sets the ephemeral_storage_requests of this ModelKernelClusterNode.


        :param ephemeral_storage_requests: The ephemeral_storage_requests of this ModelKernelClusterNode.  # noqa: E501
        :type ephemeral_storage_requests: int
        """

        self._ephemeral_storage_requests = ephemeral_storage_requests

    @property
    def gpu_allocatable(self):
        """Gets the gpu_allocatable of this ModelKernelClusterNode.  # noqa: E501


        :return: The gpu_allocatable of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._gpu_allocatable

    @gpu_allocatable.setter
    def gpu_allocatable(self, gpu_allocatable):
        """Sets the gpu_allocatable of this ModelKernelClusterNode.


        :param gpu_allocatable: The gpu_allocatable of this ModelKernelClusterNode.  # noqa: E501
        :type gpu_allocatable: int
        """

        self._gpu_allocatable = gpu_allocatable

    @property
    def gpu_limits(self):
        """Gets the gpu_limits of this ModelKernelClusterNode.  # noqa: E501


        :return: The gpu_limits of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._gpu_limits

    @gpu_limits.setter
    def gpu_limits(self, gpu_limits):
        """Sets the gpu_limits of this ModelKernelClusterNode.


        :param gpu_limits: The gpu_limits of this ModelKernelClusterNode.  # noqa: E501
        :type gpu_limits: int
        """

        self._gpu_limits = gpu_limits

    @property
    def gpu_product_name(self):
        """Gets the gpu_product_name of this ModelKernelClusterNode.  # noqa: E501


        :return: The gpu_product_name of this ModelKernelClusterNode.  # noqa: E501
        :rtype: str
        """
        return self._gpu_product_name

    @gpu_product_name.setter
    def gpu_product_name(self, gpu_product_name):
        """Sets the gpu_product_name of this ModelKernelClusterNode.


        :param gpu_product_name: The gpu_product_name of this ModelKernelClusterNode.  # noqa: E501
        :type gpu_product_name: str
        """

        self._gpu_product_name = gpu_product_name

    @property
    def gpu_requests(self):
        """Gets the gpu_requests of this ModelKernelClusterNode.  # noqa: E501


        :return: The gpu_requests of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._gpu_requests

    @gpu_requests.setter
    def gpu_requests(self, gpu_requests):
        """Sets the gpu_requests of this ModelKernelClusterNode.


        :param gpu_requests: The gpu_requests of this ModelKernelClusterNode.  # noqa: E501
        :type gpu_requests: int
        """

        self._gpu_requests = gpu_requests

    @property
    def id(self):
        """Gets the id of this ModelKernelClusterNode.  # noqa: E501


        :return: The id of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ModelKernelClusterNode.


        :param id: The id of this ModelKernelClusterNode.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def memory_allocatable(self):
        """Gets the memory_allocatable of this ModelKernelClusterNode.  # noqa: E501


        :return: The memory_allocatable of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._memory_allocatable

    @memory_allocatable.setter
    def memory_allocatable(self, memory_allocatable):
        """Sets the memory_allocatable of this ModelKernelClusterNode.


        :param memory_allocatable: The memory_allocatable of this ModelKernelClusterNode.  # noqa: E501
        :type memory_allocatable: int
        """

        self._memory_allocatable = memory_allocatable

    @property
    def memory_limits(self):
        """Gets the memory_limits of this ModelKernelClusterNode.  # noqa: E501


        :return: The memory_limits of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._memory_limits

    @memory_limits.setter
    def memory_limits(self, memory_limits):
        """Sets the memory_limits of this ModelKernelClusterNode.


        :param memory_limits: The memory_limits of this ModelKernelClusterNode.  # noqa: E501
        :type memory_limits: int
        """

        self._memory_limits = memory_limits

    @property
    def memory_requests(self):
        """Gets the memory_requests of this ModelKernelClusterNode.  # noqa: E501


        :return: The memory_requests of this ModelKernelClusterNode.  # noqa: E501
        :rtype: int
        """
        return self._memory_requests

    @memory_requests.setter
    def memory_requests(self, memory_requests):
        """Sets the memory_requests of this ModelKernelClusterNode.


        :param memory_requests: The memory_requests of this ModelKernelClusterNode.  # noqa: E501
        :type memory_requests: int
        """

        self._memory_requests = memory_requests

    @property
    def name(self):
        """Gets the name of this ModelKernelClusterNode.  # noqa: E501


        :return: The name of this ModelKernelClusterNode.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ModelKernelClusterNode.


        :param name: The name of this ModelKernelClusterNode.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ModelKernelClusterNode.  # noqa: E501


        :return: The updated_dt of this ModelKernelClusterNode.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ModelKernelClusterNode.


        :param updated_dt: The updated_dt of this ModelKernelClusterNode.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelKernelClusterNode):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ModelKernelClusterNode):
            return True

        return self.to_dict() != other.to_dict()



import collections
import numbers

import six

from openapi_client import ResponseExperimentInfo
from savvihub import history
from savvihub.exceptions import ArgumentException


class Experiment:
    def __init__(self, experiment: ResponseExperimentInfo, client):
        self.id = experiment.id
        self.client = client
        self._history = None

    @classmethod
    def from_given(cls, experiment_id, client):
        return cls(client.experiment_id_read(experiment_id), client)

    @property
    def history(self):
        if not self._history:
            self._history = history.History(self)
        return self._history

    def log(self, row, *, step=None):
        if not isinstance(row, collections.Mapping):
            raise ArgumentException(".log() takes a dictionary as a parameter")

        if any(not isinstance(key, six.string_types) for key in row.keys()):
            raise ArgumentException("The key of dictionary in .log() parameter must be str")

        for k in row.keys():
            if not k:
                raise ArgumentException("Logging empty key is not supported")

        if not isinstance(step, numbers.Number):
            raise ArgumentException(f"Step must be a number, not {type(step)}")
        if step is None or step < 0:
            raise ArgumentException(f"Step must be a positive integer, not {step}")
        if not isinstance(type(step), int):
            step = int(round(step))

        self.history.update(self.client, row, step)



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseServiceTemplateInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_by_id': 'int',
        'created_dt': 'datetime',
        'description': 'str',
        'env_vars': 'ModelEnvVars',
        'id': 'int',
        'image_execute_type': 'str',
        'is_public': 'bool',
        'is_savvihub_managed': 'bool',
        'kernel_image': 'ResponseKernelImage',
        'kernel_image_id': 'int',
        'kernel_resource_spec': 'ResponseKernelResourceSpec',
        'kernel_resource_spec_id': 'int',
        'name': 'str',
        'ports': 'ModelServicePorts',
        'service_config': 'ModelServiceConfig',
        'service_type': 'str',
        'start_command': 'str',
        'updated_dt': 'datetime',
        'workspace_id': 'int'
    }

    attribute_map = {
        'created_by_id': 'created_by_id',
        'created_dt': 'created_dt',
        'description': 'description',
        'env_vars': 'env_vars',
        'id': 'id',
        'image_execute_type': 'image_execute_type',
        'is_public': 'is_public',
        'is_savvihub_managed': 'is_savvihub_managed',
        'kernel_image': 'kernel_image',
        'kernel_image_id': 'kernel_image_id',
        'kernel_resource_spec': 'kernel_resource_spec',
        'kernel_resource_spec_id': 'kernel_resource_spec_id',
        'name': 'name',
        'ports': 'ports',
        'service_config': 'service_config',
        'service_type': 'service_type',
        'start_command': 'start_command',
        'updated_dt': 'updated_dt',
        'workspace_id': 'workspace_id'
    }

    def __init__(self, created_by_id=None, created_dt=None, description=None, env_vars=None, id=None, image_execute_type=None, is_public=None, is_savvihub_managed=None, kernel_image=None, kernel_image_id=None, kernel_resource_spec=None, kernel_resource_spec_id=None, name=None, ports=None, service_config=None, service_type=None, start_command=None, updated_dt=None, workspace_id=None, local_vars_configuration=None):  # noqa: E501
        """ResponseServiceTemplateInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_by_id = None
        self._created_dt = None
        self._description = None
        self._env_vars = None
        self._id = None
        self._image_execute_type = None
        self._is_public = None
        self._is_savvihub_managed = None
        self._kernel_image = None
        self._kernel_image_id = None
        self._kernel_resource_spec = None
        self._kernel_resource_spec_id = None
        self._name = None
        self._ports = None
        self._service_config = None
        self._service_type = None
        self._start_command = None
        self._updated_dt = None
        self._workspace_id = None
        self.discriminator = None

        self.created_by_id = created_by_id
        self.created_dt = created_dt
        self.description = description
        if env_vars is not None:
            self.env_vars = env_vars
        self.id = id
        self.image_execute_type = image_execute_type
        self.is_public = is_public
        self.is_savvihub_managed = is_savvihub_managed
        if kernel_image is not None:
            self.kernel_image = kernel_image
        self.kernel_image_id = kernel_image_id
        if kernel_resource_spec is not None:
            self.kernel_resource_spec = kernel_resource_spec
        self.kernel_resource_spec_id = kernel_resource_spec_id
        self.name = name
        if ports is not None:
            self.ports = ports
        if service_config is not None:
            self.service_config = service_config
        self.service_type = service_type
        self.start_command = start_command
        self.updated_dt = updated_dt
        self.workspace_id = workspace_id

    @property
    def created_by_id(self):
        """Gets the created_by_id of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The created_by_id of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: int
        """
        return self._created_by_id

    @created_by_id.setter
    def created_by_id(self, created_by_id):
        """Sets the created_by_id of this ResponseServiceTemplateInfo.


        :param created_by_id: The created_by_id of this ResponseServiceTemplateInfo.  # noqa: E501
        :type created_by_id: int
        """
        if self.local_vars_configuration.client_side_validation and created_by_id is None:  # noqa: E501
            raise ValueError("Invalid value for `created_by_id`, must not be `None`")  # noqa: E501

        self._created_by_id = created_by_id

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The created_dt of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseServiceTemplateInfo.


        :param created_dt: The created_dt of this ResponseServiceTemplateInfo.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def description(self):
        """Gets the description of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The description of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ResponseServiceTemplateInfo.


        :param description: The description of this ResponseServiceTemplateInfo.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def env_vars(self):
        """Gets the env_vars of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The env_vars of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: ModelEnvVars
        """
        return self._env_vars

    @env_vars.setter
    def env_vars(self, env_vars):
        """Sets the env_vars of this ResponseServiceTemplateInfo.


        :param env_vars: The env_vars of this ResponseServiceTemplateInfo.  # noqa: E501
        :type env_vars: ModelEnvVars
        """

        self._env_vars = env_vars

    @property
    def id(self):
        """Gets the id of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The id of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseServiceTemplateInfo.


        :param id: The id of this ResponseServiceTemplateInfo.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def image_execute_type(self):
        """Gets the image_execute_type of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The image_execute_type of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: str
        """
        return self._image_execute_type

    @image_execute_type.setter
    def image_execute_type(self, image_execute_type):
        """Sets the image_execute_type of this ResponseServiceTemplateInfo.


        :param image_execute_type: The image_execute_type of this ResponseServiceTemplateInfo.  # noqa: E501
        :type image_execute_type: str
        """

        self._image_execute_type = image_execute_type

    @property
    def is_public(self):
        """Gets the is_public of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The is_public of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: bool
        """
        return self._is_public

    @is_public.setter
    def is_public(self, is_public):
        """Sets the is_public of this ResponseServiceTemplateInfo.


        :param is_public: The is_public of this ResponseServiceTemplateInfo.  # noqa: E501
        :type is_public: bool
        """
        if self.local_vars_configuration.client_side_validation and is_public is None:  # noqa: E501
            raise ValueError("Invalid value for `is_public`, must not be `None`")  # noqa: E501

        self._is_public = is_public

    @property
    def is_savvihub_managed(self):
        """Gets the is_savvihub_managed of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The is_savvihub_managed of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: bool
        """
        return self._is_savvihub_managed

    @is_savvihub_managed.setter
    def is_savvihub_managed(self, is_savvihub_managed):
        """Sets the is_savvihub_managed of this ResponseServiceTemplateInfo.


        :param is_savvihub_managed: The is_savvihub_managed of this ResponseServiceTemplateInfo.  # noqa: E501
        :type is_savvihub_managed: bool
        """
        if self.local_vars_configuration.client_side_validation and is_savvihub_managed is None:  # noqa: E501
            raise ValueError("Invalid value for `is_savvihub_managed`, must not be `None`")  # noqa: E501

        self._is_savvihub_managed = is_savvihub_managed

    @property
    def kernel_image(self):
        """Gets the kernel_image of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The kernel_image of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: ResponseKernelImage
        """
        return self._kernel_image

    @kernel_image.setter
    def kernel_image(self, kernel_image):
        """Sets the kernel_image of this ResponseServiceTemplateInfo.


        :param kernel_image: The kernel_image of this ResponseServiceTemplateInfo.  # noqa: E501
        :type kernel_image: ResponseKernelImage
        """

        self._kernel_image = kernel_image

    @property
    def kernel_image_id(self):
        """Gets the kernel_image_id of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The kernel_image_id of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: int
        """
        return self._kernel_image_id

    @kernel_image_id.setter
    def kernel_image_id(self, kernel_image_id):
        """Sets the kernel_image_id of this ResponseServiceTemplateInfo.


        :param kernel_image_id: The kernel_image_id of this ResponseServiceTemplateInfo.  # noqa: E501
        :type kernel_image_id: int
        """

        self._kernel_image_id = kernel_image_id

    @property
    def kernel_resource_spec(self):
        """Gets the kernel_resource_spec of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The kernel_resource_spec of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: ResponseKernelResourceSpec
        """
        return self._kernel_resource_spec

    @kernel_resource_spec.setter
    def kernel_resource_spec(self, kernel_resource_spec):
        """Sets the kernel_resource_spec of this ResponseServiceTemplateInfo.


        :param kernel_resource_spec: The kernel_resource_spec of this ResponseServiceTemplateInfo.  # noqa: E501
        :type kernel_resource_spec: ResponseKernelResourceSpec
        """

        self._kernel_resource_spec = kernel_resource_spec

    @property
    def kernel_resource_spec_id(self):
        """Gets the kernel_resource_spec_id of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The kernel_resource_spec_id of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: int
        """
        return self._kernel_resource_spec_id

    @kernel_resource_spec_id.setter
    def kernel_resource_spec_id(self, kernel_resource_spec_id):
        """Sets the kernel_resource_spec_id of this ResponseServiceTemplateInfo.


        :param kernel_resource_spec_id: The kernel_resource_spec_id of this ResponseServiceTemplateInfo.  # noqa: E501
        :type kernel_resource_spec_id: int
        """

        self._kernel_resource_spec_id = kernel_resource_spec_id

    @property
    def name(self):
        """Gets the name of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The name of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ResponseServiceTemplateInfo.


        :param name: The name of this ResponseServiceTemplateInfo.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def ports(self):
        """Gets the ports of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The ports of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: ModelServicePorts
        """
        return self._ports

    @ports.setter
    def ports(self, ports):
        """Sets the ports of this ResponseServiceTemplateInfo.


        :param ports: The ports of this ResponseServiceTemplateInfo.  # noqa: E501
        :type ports: ModelServicePorts
        """

        self._ports = ports

    @property
    def service_config(self):
        """Gets the service_config of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The service_config of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: ModelServiceConfig
        """
        return self._service_config

    @service_config.setter
    def service_config(self, service_config):
        """Sets the service_config of this ResponseServiceTemplateInfo.


        :param service_config: The service_config of this ResponseServiceTemplateInfo.  # noqa: E501
        :type service_config: ModelServiceConfig
        """

        self._service_config = service_config

    @property
    def service_type(self):
        """Gets the service_type of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The service_type of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: str
        """
        return self._service_type

    @service_type.setter
    def service_type(self, service_type):
        """Sets the service_type of this ResponseServiceTemplateInfo.


        :param service_type: The service_type of this ResponseServiceTemplateInfo.  # noqa: E501
        :type service_type: str
        """
        if self.local_vars_configuration.client_side_validation and service_type is None:  # noqa: E501
            raise ValueError("Invalid value for `service_type`, must not be `None`")  # noqa: E501

        self._service_type = service_type

    @property
    def start_command(self):
        """Gets the start_command of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The start_command of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: str
        """
        return self._start_command

    @start_command.setter
    def start_command(self, start_command):
        """Sets the start_command of this ResponseServiceTemplateInfo.


        :param start_command: The start_command of this ResponseServiceTemplateInfo.  # noqa: E501
        :type start_command: str
        """

        self._start_command = start_command

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The updated_dt of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseServiceTemplateInfo.


        :param updated_dt: The updated_dt of this ResponseServiceTemplateInfo.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def workspace_id(self):
        """Gets the workspace_id of this ResponseServiceTemplateInfo.  # noqa: E501


        :return: The workspace_id of this ResponseServiceTemplateInfo.  # noqa: E501
        :rtype: int
        """
        return self._workspace_id

    @workspace_id.setter
    def workspace_id(self, workspace_id):
        """Sets the workspace_id of this ResponseServiceTemplateInfo.


        :param workspace_id: The workspace_id of this ResponseServiceTemplateInfo.  # noqa: E501
        :type workspace_id: int
        """
        if self.local_vars_configuration.client_side_validation and workspace_id is None:  # noqa: E501
            raise ValueError("Invalid value for `workspace_id`, must not be `None`")  # noqa: E501

        self._workspace_id = workspace_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseServiceTemplateInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseServiceTemplateInfo):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseKernelResourceSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cpu_limit': 'float',
        'cpu_type': 'str',
        'created_dt': 'datetime',
        'description': 'str',
        'gpu_limit': 'int',
        'gpu_type': 'str',
        'id': 'int',
        'memory_limit': 'str',
        'name': 'str',
        'processor_type': 'str',
        'spot': 'bool',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'cpu_limit': 'cpu_limit',
        'cpu_type': 'cpu_type',
        'created_dt': 'created_dt',
        'description': 'description',
        'gpu_limit': 'gpu_limit',
        'gpu_type': 'gpu_type',
        'id': 'id',
        'memory_limit': 'memory_limit',
        'name': 'name',
        'processor_type': 'processor_type',
        'spot': 'spot',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, cpu_limit=None, cpu_type=None, created_dt=None, description=None, gpu_limit=None, gpu_type=None, id=None, memory_limit=None, name=None, processor_type=None, spot=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """ResponseKernelResourceSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cpu_limit = None
        self._cpu_type = None
        self._created_dt = None
        self._description = None
        self._gpu_limit = None
        self._gpu_type = None
        self._id = None
        self._memory_limit = None
        self._name = None
        self._processor_type = None
        self._spot = None
        self._updated_dt = None
        self.discriminator = None

        self.cpu_limit = cpu_limit
        self.cpu_type = cpu_type
        self.created_dt = created_dt
        self.description = description
        self.gpu_limit = gpu_limit
        self.gpu_type = gpu_type
        self.id = id
        self.memory_limit = memory_limit
        self.name = name
        self.processor_type = processor_type
        self.spot = spot
        self.updated_dt = updated_dt

    @property
    def cpu_limit(self):
        """Gets the cpu_limit of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The cpu_limit of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: float
        """
        return self._cpu_limit

    @cpu_limit.setter
    def cpu_limit(self, cpu_limit):
        """Sets the cpu_limit of this ResponseKernelResourceSpec.


        :param cpu_limit: The cpu_limit of this ResponseKernelResourceSpec.  # noqa: E501
        :type cpu_limit: float
        """
        if self.local_vars_configuration.client_side_validation and cpu_limit is None:  # noqa: E501
            raise ValueError("Invalid value for `cpu_limit`, must not be `None`")  # noqa: E501

        self._cpu_limit = cpu_limit

    @property
    def cpu_type(self):
        """Gets the cpu_type of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The cpu_type of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: str
        """
        return self._cpu_type

    @cpu_type.setter
    def cpu_type(self, cpu_type):
        """Sets the cpu_type of this ResponseKernelResourceSpec.


        :param cpu_type: The cpu_type of this ResponseKernelResourceSpec.  # noqa: E501
        :type cpu_type: str
        """
        if self.local_vars_configuration.client_side_validation and cpu_type is None:  # noqa: E501
            raise ValueError("Invalid value for `cpu_type`, must not be `None`")  # noqa: E501

        self._cpu_type = cpu_type

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The created_dt of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseKernelResourceSpec.


        :param created_dt: The created_dt of this ResponseKernelResourceSpec.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def description(self):
        """Gets the description of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The description of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ResponseKernelResourceSpec.


        :param description: The description of this ResponseKernelResourceSpec.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def gpu_limit(self):
        """Gets the gpu_limit of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The gpu_limit of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: int
        """
        return self._gpu_limit

    @gpu_limit.setter
    def gpu_limit(self, gpu_limit):
        """Sets the gpu_limit of this ResponseKernelResourceSpec.


        :param gpu_limit: The gpu_limit of this ResponseKernelResourceSpec.  # noqa: E501
        :type gpu_limit: int
        """
        if self.local_vars_configuration.client_side_validation and gpu_limit is None:  # noqa: E501
            raise ValueError("Invalid value for `gpu_limit`, must not be `None`")  # noqa: E501

        self._gpu_limit = gpu_limit

    @property
    def gpu_type(self):
        """Gets the gpu_type of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The gpu_type of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: str
        """
        return self._gpu_type

    @gpu_type.setter
    def gpu_type(self, gpu_type):
        """Sets the gpu_type of this ResponseKernelResourceSpec.


        :param gpu_type: The gpu_type of this ResponseKernelResourceSpec.  # noqa: E501
        :type gpu_type: str
        """
        if self.local_vars_configuration.client_side_validation and gpu_type is None:  # noqa: E501
            raise ValueError("Invalid value for `gpu_type`, must not be `None`")  # noqa: E501

        self._gpu_type = gpu_type

    @property
    def id(self):
        """Gets the id of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The id of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseKernelResourceSpec.


        :param id: The id of this ResponseKernelResourceSpec.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def memory_limit(self):
        """Gets the memory_limit of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The memory_limit of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: str
        """
        return self._memory_limit

    @memory_limit.setter
    def memory_limit(self, memory_limit):
        """Sets the memory_limit of this ResponseKernelResourceSpec.


        :param memory_limit: The memory_limit of this ResponseKernelResourceSpec.  # noqa: E501
        :type memory_limit: str
        """
        if self.local_vars_configuration.client_side_validation and memory_limit is None:  # noqa: E501
            raise ValueError("Invalid value for `memory_limit`, must not be `None`")  # noqa: E501

        self._memory_limit = memory_limit

    @property
    def name(self):
        """Gets the name of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The name of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ResponseKernelResourceSpec.


        :param name: The name of this ResponseKernelResourceSpec.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def processor_type(self):
        """Gets the processor_type of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The processor_type of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: str
        """
        return self._processor_type

    @processor_type.setter
    def processor_type(self, processor_type):
        """Sets the processor_type of this ResponseKernelResourceSpec.


        :param processor_type: The processor_type of this ResponseKernelResourceSpec.  # noqa: E501
        :type processor_type: str
        """
        if self.local_vars_configuration.client_side_validation and processor_type is None:  # noqa: E501
            raise ValueError("Invalid value for `processor_type`, must not be `None`")  # noqa: E501

        self._processor_type = processor_type

    @property
    def spot(self):
        """Gets the spot of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The spot of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: bool
        """
        return self._spot

    @spot.setter
    def spot(self, spot):
        """Sets the spot of this ResponseKernelResourceSpec.


        :param spot: The spot of this ResponseKernelResourceSpec.  # noqa: E501
        :type spot: bool
        """
        if self.local_vars_configuration.client_side_validation and spot is None:  # noqa: E501
            raise ValueError("Invalid value for `spot`, must not be `None`")  # noqa: E501

        self._spot = spot

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseKernelResourceSpec.  # noqa: E501


        :return: The updated_dt of this ResponseKernelResourceSpec.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseKernelResourceSpec.


        :param updated_dt: The updated_dt of this ResponseKernelResourceSpec.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseKernelResourceSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseKernelResourceSpec):
            return True

        return self.to_dict() != other.to_dict()



import os
import time
from collections import defaultdict
from urllib.parse import urlparse

import typer
from terminaltables import AsciiTable

from openapi_client import ResponseVolume
from savvihub.api.exceptions import NoSuchFileOrDirectoryAPIException
from savvihub.api.uploader import Downloader, Uploader
from savvihub.cli.exceptions import ExitException
from savvihub.cli.typer import Typer, Context
from savvihub.common.utils import sizeof_fmt, remove_suffix

volume_app = Typer()


def refine_volume_path(
    path_arg,
    raise_if_not_empty=False,
    raise_if_not_dir=False,
    raise_if_not_exist=False,
    remote_only=False,
):
    if path_arg.startswith('svds://'):
        raise ExitException('[Error] Use \'sv://\' for volumes.')

    if path_arg.startswith('sv://'):
        return path_arg, True
    elif remote_only:
        return 'sv://' + path_arg, True

    path = os.path.abspath(path_arg)
    if os.path.exists(path):
        if not os.path.isdir(path) and raise_if_not_dir:
            raise ExitException(f'Must specify directory: {path_arg}')
        if raise_if_not_empty and len(os.listdir(path)) > 0:
            raise ExitException(f'Must specify empty directory: {path_arg}')
    else:
        if raise_if_not_exist:
            raise ExitException(f'Must specify directory: {path_arg}')

    return path, False


def parse_remote_volume_path(remote_path: str) -> (ResponseVolume, str, str):
    # sv://1:snapshot/path/to/files
    u = urlparse(remote_path)
    volume_snapshot = u.netloc
    split_snapshot = volume_snapshot.split(':', 1)
    volume_id = int(split_snapshot[0])
    file_path = u.path.lstrip('/')

    snapshot = split_snapshot[1] if len(split_snapshot) == 2 else 'latest'

    return volume_id, snapshot, file_path


@volume_app.callback()
def main():
    """
    Manage files in the volume
    """


@volume_app.command(auth_required=True)
def ls(
    ctx: Context,
    source_path_arg: str = typer.Argument(...),
    recursive: bool = typer.Option(False, '-r', '--recursive', help='recursive flag'),
    directory: bool = typer.Option(False, '-d', '--directory', help='list the directory itself, not its contents')
):
    """
    List files in the dataset with prefix
    """
    source_path, is_source_remote = refine_volume_path(source_path_arg, raise_if_not_exist=True, remote_only=True)

    volume_id, snapshot, path = parse_remote_volume_path(source_path)
    volume_file_list(ctx, volume_id, snapshot, path, recursive, directory)


@volume_app.command(auth_required=True)
def rm(
    ctx: Context,
    source_path_arg: str = typer.Argument(...),
    recursive: bool = typer.Option(False, '-r', '-R', '--recursive',
                                   help='Remove directories and their contents recursively'),
):
    """
    Remove files in the volume with path
    """
    source_path, is_source_remote = refine_volume_path(source_path_arg, raise_if_not_exist=True, remote_only=True)

    volume_id, snapshot, path = parse_remote_volume_path(source_path)
    volume_file_remove(ctx, volume_id, path, recursive)


@volume_app.command(auth_required=True)
def cp(
    ctx: Context,
    source_path_arg: str = typer.Argument(...),
    dest_path_arg: str = typer.Argument(...),
    recursive: bool = typer.Option(False, '-r', '--recursive'),
    watch: bool = typer.Option(False, '-w', '--watch'),
):
    source_path, is_source_remote = refine_volume_path(source_path_arg, raise_if_not_exist=True)
    dest_path, is_dest_remote = refine_volume_path(dest_path_arg)

    source_volume_id = dest_volume_id = None
    source_snapshot = dest_snapshot = None
    if is_source_remote:
        source_volume_id, source_snapshot, source_path = parse_remote_volume_path(source_path)
    if is_dest_remote:
        dest_volume_id, dest_snapshot, dest_path = parse_remote_volume_path(dest_path)

    volume_file_copy(ctx, source_volume_id, source_snapshot, source_path,
                     dest_volume_id, dest_snapshot, dest_path,
                     recursive, watch)


def volume_file_list(ctx: Context, volume_id: int, snapshot: str, path: str, recursive: bool, directory: bool):
    client = ctx.authenticated_client

    if recursive and directory:
        raise ExitException('[Error] -r and -d options cannot be used in one command')
    elif recursive:
        files = client.volume_file_list(volume_id, snapshot=snapshot, path=path, recursive=True).results
    elif directory:
        if not path:
            raise ExitException('[Error] path must be specified to run -d option')

        file = client.volume_file_read(volume_id, path, snapshot)
        files = [file] if file is not None else None
    else:
        files = client.volume_file_list(volume_id, snapshot=snapshot, path=path, recursive=False).results

    if files is None:
        raise ExitException('No such file or directory.')

    if len(files) == 0:
        raise ExitException('This directory is empty.', exit_code=0)

    print_file_list(files, recursive)


def volume_file_remove(ctx: Context, volume_id: int, path: str, recursive: bool):
    client = ctx.authenticated_client
    try:
        file = client.volume_file_read(volume_id, path, 'latest')
    except NoSuchFileOrDirectoryAPIException:
        raise ExitException('[Error] Request entity not found')

    if file.is_dir:
        if not recursive:
            raise ExitException('[Error] Remove directory should use -r option')

        deleted_files = client.volume_file_delete(volume_id, path, True).deleted_files
    else:
        deleted_files = client.volume_file_delete(volume_id, path, False).deleted_files

    if not deleted_files:
        raise ExitException('[Error] Server error')

    typer.echo('Successfully deleted files:')
    print_file_list(deleted_files, True)


def print_file_list(files, recursive):
    files = sorted(files, key=lambda f: f.path if recursive else (not f.is_dir, f.path))

    table_content = []
    for file in files:
        row = [file.path if recursive else file.path.rsplit('/', 1)[-1],
               '' if file.is_dir else sizeof_fmt(file.size)]
        if file.is_dir:
            row[0] += '/'
        table_content.append(row)

    table = AsciiTable(table_content)
    table.inner_column_border = False
    table.inner_heading_row_border = False
    table.inner_footing_row_border = False
    table.outer_border = False

    typer.echo(table.table)


def volume_file_copy(ctx: Context, source_volume_id, source_snapshot, source_path, dest_volume_id, dest_snapshot, dest_path, recursive, watch):
    hashmap = defaultdict(lambda: '')
    client = ctx.authenticated_client

    while True:
        if source_volume_id is not None and dest_volume_id is not None:
            # remote -> remote
            if source_volume_id != dest_volume_id:
                raise ExitException('Currently files can be copied only within the same volume.')

            if dest_snapshot != 'latest':
                raise ExitException(f'Cannot write to snapshots: {dest_path}')

            source_file = client.volume_file_read(source_volume_id, source_path, source_snapshot)
            if source_file is None:
                raise ExitException(f'No such file or directory: {source_path}')

            if source_file.is_dir and not recursive:
                raise ExitException(f'Source path is a directory, you should call with --recursive option.')

            try:
                dest_file = client.volume_file_read(dest_volume_id, dest_path, 'latest')
            except NoSuchFileOrDirectoryAPIException:
                pass
            else:
                if source_file.is_dir and not dest_file.is_dir:
                    raise ExitException(f'Source path is a directory, but destination path is a file.')
            finally:
                resp = client.volume_file_copy(
                    source_volume_id, source_path, source_snapshot,
                    dest_path, recursive=recursive,
                )
                typer.echo('Successfully copied files:')
                print_file_list(resp.copied_files, True)

        elif source_volume_id is not None and dest_volume_id is None:
            # remote -> local
            try:
                source_file = client.volume_file_read(source_volume_id, source_path, source_snapshot)
            except NoSuchFileOrDirectoryAPIException:
                raise ExitException(f'Source file does not exist: {source_path}')

            if source_file.is_dir:
                if not recursive:
                    raise ExitException(f'Source path is a directory, you should call with --recursive option.')

                if os.path.exists(dest_path) and not os.path.isdir(dest_path):
                    raise ExitException(f'Destination path is not a directory: {dest_path}')

                # download directory
                typer.echo('Fetching file metadata...')
                files = client.volume_file_list(
                    source_volume_id,
                    path=source_path,
                    snapshot=source_snapshot,
                    recursive=True,
                    need_download_url=True,
                ).results
                files = [file for file in files if hashmap[file.path] != file.hash]

                if source_path.endswith('/'):
                    for file in files:
                        file.path = file.path.replace(source_path, '', 1)
                else:
                    dir_name = os.path.dirname(source_path)
                    for file in files:
                        if dir_name:
                            file.path = file.path.replace(dir_name + '/', '', 1)

                typer.echo(f'Found {len(files)} files to download.')
                if len(files) > 0:
                    typer.echo('Downloading...')
                    Downloader.bulk_download(dest_path, files, progressable=typer.progressbar)
                    for file in files:
                        hashmap[file.path] = file.hash
                    typer.echo('Successfully downloaded.')
            else:
                # download file
                dest_file_path = dest_path
                if os.path.isdir(dest_path):
                    dest_file_path = os.path.join(dest_path, os.path.basename(source_file.path))

                Downloader.download(dest_file_path, source_file, progressable=typer.progressbar)
                hashmap[source_file.path] = source_file.hash
                typer.echo('Successfully downloaded.')

        elif source_volume_id is None and dest_volume_id is not None:
            # local -> remote
            if not os.path.exists(source_path):
                raise ExitException(f'No such file or directory: {source_path}')

            if dest_snapshot != 'latest':
                raise ExitException(f'Cannot write to snapshots: {dest_path}')

            try:
                dest_file = client.volume_file_read(dest_volume_id, dest_path, dest_snapshot)
            except NoSuchFileOrDirectoryAPIException:
                dest_file = None

            if os.path.isdir(source_path):
                if not recursive:
                    raise ExitException(f'Source path is a directory, you should call with --recursive option.')

                files = Uploader.get_files_to_upload(source_path, hashmap)
                if dest_file is None:
                    # x/y/dir1/ -> c/ (not exist)
                    # x/y/dir1/file1/ -> c/file1
                    dest_base_path = dest_path
                elif dest_file.is_dir:
                    # x/y/dir1/ -> c/ (exist path)
                    # x/y/dir1/file1/ -> c/dir1/file1
                    if source_path.endswith('/'):
                        dest_base_path = dest_path
                    else:
                        dest_base_path = os.path.join(dest_path, os.path.basename(remove_suffix(source_path, "/")))
                else:
                    raise ExitException(f'Destination path is not a directory: {dest_path}')

                typer.echo(f'Found {len(files)} files to upload.')
                if len(files) > 0:
                    typer.echo('Uploading...')
                    Uploader.bulk_upload(ctx, source_path, files, dest_volume_id, dest_base_path, progressable=typer.progressbar)
                    hashmap = Uploader.get_hashmap(source_path)
                    typer.echo('Successfully uploaded.')
            else:
                # x/y/dir1/file1 ->
                if dest_file is None:
                    if dest_path.endswith('/'):
                        Uploader.upload(ctx, source_path, dest_volume_id, os.path.join(dest_path, os.path.basename(source_path)))
                    else:
                        Uploader.upload(ctx, source_path, dest_volume_id, dest_path)
                elif dest_file.is_dir:
                    Uploader.upload(ctx, source_path, dest_volume_id, os.path.join(dest_path, os.path.basename(source_path)))
                else:
                    Uploader.upload(ctx, source_path, dest_volume_id, dest_path)
                typer.echo('Successfully uploaded.')

        else:
            typer.echo('Use \'svds://\' scheme for remote files')
            return

        if not watch:
            return

        time.sleep(10)



import keras

from savvihub import log


class SavviHubCallback(keras.callbacks.Callback):
    def on_epoch_end(self, epoch, logs=None):
        log(
            step=epoch,
            row={'loss': logs.get('loss'), 'accuracy': logs.get('accuracy')}
        )
        print(f'epoch: {epoch}, logs: {logs}')



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


import inspect
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponseWorkspaceMember(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_dt': 'datetime',
        'email': 'str',
        'id': 'int',
        'is_email_verified': 'bool',
        'last_login': 'datetime',
        'name': 'str',
        'permission': 'str',
        'status': 'str',
        'updated_dt': 'datetime',
        'username': 'str'
    }

    attribute_map = {
        'created_dt': 'created_dt',
        'email': 'email',
        'id': 'id',
        'is_email_verified': 'is_email_verified',
        'last_login': 'last_login',
        'name': 'name',
        'permission': 'permission',
        'status': 'status',
        'updated_dt': 'updated_dt',
        'username': 'username'
    }

    def __init__(self, created_dt=None, email=None, id=None, is_email_verified=None, last_login=None, name=None, permission=None, status=None, updated_dt=None, username=None, local_vars_configuration=None):  # noqa: E501
        """ResponseWorkspaceMember - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_dt = None
        self._email = None
        self._id = None
        self._is_email_verified = None
        self._last_login = None
        self._name = None
        self._permission = None
        self._status = None
        self._updated_dt = None
        self._username = None
        self.discriminator = None

        self.created_dt = created_dt
        self.email = email
        self.id = id
        self.is_email_verified = is_email_verified
        self.last_login = last_login
        self.name = name
        self.permission = permission
        self.status = status
        self.updated_dt = updated_dt
        self.username = username

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponseWorkspaceMember.  # noqa: E501


        :return: The created_dt of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponseWorkspaceMember.


        :param created_dt: The created_dt of this ResponseWorkspaceMember.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def email(self):
        """Gets the email of this ResponseWorkspaceMember.  # noqa: E501


        :return: The email of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this ResponseWorkspaceMember.


        :param email: The email of this ResponseWorkspaceMember.  # noqa: E501
        :type email: str
        """
        if self.local_vars_configuration.client_side_validation and email is None:  # noqa: E501
            raise ValueError("Invalid value for `email`, must not be `None`")  # noqa: E501

        self._email = email

    @property
    def id(self):
        """Gets the id of this ResponseWorkspaceMember.  # noqa: E501


        :return: The id of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResponseWorkspaceMember.


        :param id: The id of this ResponseWorkspaceMember.  # noqa: E501
        :type id: int
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def is_email_verified(self):
        """Gets the is_email_verified of this ResponseWorkspaceMember.  # noqa: E501


        :return: The is_email_verified of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: bool
        """
        return self._is_email_verified

    @is_email_verified.setter
    def is_email_verified(self, is_email_verified):
        """Sets the is_email_verified of this ResponseWorkspaceMember.


        :param is_email_verified: The is_email_verified of this ResponseWorkspaceMember.  # noqa: E501
        :type is_email_verified: bool
        """
        if self.local_vars_configuration.client_side_validation and is_email_verified is None:  # noqa: E501
            raise ValueError("Invalid value for `is_email_verified`, must not be `None`")  # noqa: E501

        self._is_email_verified = is_email_verified

    @property
    def last_login(self):
        """Gets the last_login of this ResponseWorkspaceMember.  # noqa: E501


        :return: The last_login of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: datetime
        """
        return self._last_login

    @last_login.setter
    def last_login(self, last_login):
        """Sets the last_login of this ResponseWorkspaceMember.


        :param last_login: The last_login of this ResponseWorkspaceMember.  # noqa: E501
        :type last_login: datetime
        """

        self._last_login = last_login

    @property
    def name(self):
        """Gets the name of this ResponseWorkspaceMember.  # noqa: E501


        :return: The name of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ResponseWorkspaceMember.


        :param name: The name of this ResponseWorkspaceMember.  # noqa: E501
        :type name: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def permission(self):
        """Gets the permission of this ResponseWorkspaceMember.  # noqa: E501


        :return: The permission of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: str
        """
        return self._permission

    @permission.setter
    def permission(self, permission):
        """Sets the permission of this ResponseWorkspaceMember.


        :param permission: The permission of this ResponseWorkspaceMember.  # noqa: E501
        :type permission: str
        """
        if self.local_vars_configuration.client_side_validation and permission is None:  # noqa: E501
            raise ValueError("Invalid value for `permission`, must not be `None`")  # noqa: E501

        self._permission = permission

    @property
    def status(self):
        """Gets the status of this ResponseWorkspaceMember.  # noqa: E501


        :return: The status of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponseWorkspaceMember.


        :param status: The status of this ResponseWorkspaceMember.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def updated_dt(self):
        """Gets the updated_dt of this ResponseWorkspaceMember.  # noqa: E501


        :return: The updated_dt of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this ResponseWorkspaceMember.


        :param updated_dt: The updated_dt of this ResponseWorkspaceMember.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def username(self):
        """Gets the username of this ResponseWorkspaceMember.  # noqa: E501


        :return: The username of this ResponseWorkspaceMember.  # noqa: E501
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """Sets the username of this ResponseWorkspaceMember.


        :param username: The username of this ResponseWorkspaceMember.  # noqa: E501
        :type username: str
        """
        if self.local_vars_configuration.client_side_validation and username is None:  # noqa: E501
            raise ValueError("Invalid value for `username`, must not be `None`")  # noqa: E501

        self._username = username

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = inspect.getargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponseWorkspaceMember):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponseWorkspaceMember):
            return True

        return self.to_dict() != other.to_dict()



import os

from savvihub.api.savvihub import SavviHubClient
from savvihub.experiment import Experiment

experiment_context = None


def log(step, row=None):
    """Log a metric during a SavviHub experiment.

    This function must be called on the SavviHub infrastructure to log the metric.
    If not executed on SavviHub's infrastructure, this function has no effect.

    :param step: a step(positive integer) for each iteration (required)
    :param row: a dictionary to log
    """
    global experiment_context
    if experiment_context is None:
        experiment_id = os.environ.get('SAVVIHUB_EXPERIMENT_ID', None)
        access_token = os.environ.get('SAVVIHUB_ACCESS_TOKEN', None)
        if experiment_id is None or access_token is None:
            return

        client = SavviHubClient(auth_header={'Authorization': f'Token {access_token}'})
        experiment_context = Experiment.from_given(experiment_id, client)

    experiment_context.log(row=row, step=step)



import inquirer
import typer

from openapi_client import ResponseWorkspace
from savvihub import SavviHubClient


def get_default_workspace(client: SavviHubClient) -> ResponseWorkspace:
    workspaces = client.workspace_list().workspaces
    if len(workspaces) == 1:
        default_workspace = workspaces[0]
        typer.echo(f'Default workspace is automatically set to `{default_workspace.name}`.')
    else:
        default_workspace = inquirer.prompt([inquirer.List(
            'default_workspace',
            message='Select default workspace',
            choices=[(ws.name, ws) for ws in workspaces],
        )]).get('default_workspace')

        typer.echo(f'Default workspace is set to `{default_workspace.name}`.')

    return default_workspace


def find_from_inquirer(options, display, message):
    return inquirer.prompt([inquirer.List(
        "question",
        message=message,
        choices=[(f'[{i+1}] {display(option)}', option) for i, option in enumerate(options)],
    )]).get("question")


def parse_dataset(dataset_full_name):
    if '@' in dataset_full_name:
        dataset_name, snapshot_name = dataset_full_name.split('@', 1)
    else:
        dataset_name = dataset_full_name
        snapshot_name = 'latest'

    if '/' in dataset_name:
        workspace_name, dataset_name = dataset_name.split('/', 1)
    else:
        workspace_name = None

    return workspace_name, dataset_name, snapshot_name
